function [DataSet, figNr] = creaDataSetSignal(fileName1, changeParams, fileName2, do_show)
%
% function [DataSet, figNr] = creaDataSetSignal(fileName1, changeParams, fileName2, do_show)
%
% Generates Dataset for...
%
% input description
%
% fileName1   % source filename
%
% changeParams structure with fields
% changeParams.N  % length of each sequence of the dataset
% changeParams.T_star  % changePoint of each sequence of the dataset
% changeParams.period  % period expressed as a number of samples
% changeParams.changeType:
%    'source'  % change the source generating data, though the period remains the same (fileName2 required)
%    'sourceAndRescale' % change the source generating data, the period remains the same and rescale the second source to meet the same mean of the first
%    'offset'  % add an offset to the measurements (fileName2 NOT required)
%    'stackat' % stackat at T_star measurements (fileName2 NOT required)
%	 'gain'    % gain change at T_star
%    'offset+gain'
%    'sensor degradation' Add AWGN having std changeParams.sensorDegradationStd * mean flow
%
% fileName2   % source filename of the post-change (if needed)
%
% do_show     % display Dataset (for debug purposes)
%
%
% output description
%
% DataSet   % matrix containing each sequence of changeParams.N data in a row
% figNr     % number of figure used for visualization purposes (only when do_show ~= 1)
%
%
% References
%
% [1]  Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "A just-in-time adaptive classification system based on the intersection of confidence intervals rule,"
% Neural Networks, Elsevier vol. 24 (2011), pp. 791-800 doi: 10.1016/j.neunet.2011.05.012
%
%
% preprints available at http://home.deib.polimi.it/boracchi/publications.html
%
%
% Giacomo Boracchi
% Politecnico di Milano
% January 2014
% giacomo.boracchi@polimi.it

figNr = [];

if exist('fileName2', 'var') || ~isempty(fileName2)
    flowStream2 = loadFlowData(fileName2)';
elseif strcmpi(changeParams.changeType, 'source')
    error('flowStream2 argument required for change type source');
else % in case of no source or of all files there is no need
    flowStream2 = [];
end

if ~exist('do_show', 'var') || isempty(do_show)
    do_show = 0;
end

N = changeParams.N;

flowStream1 = loadFlowData(fileName1)';

% replicate the dataset when there is no source change and when DataSet has more than a row
if numel(flowStream1) > 2 * N;
    switch(lower(changeParams.changeType))
        case {'offset', 'offset+gain', 'offsetgain', 'gain+offset', 'gain', 'stackat',  'sensordegradation'}
            flowStream2 = flowStream1;
    end
end % thus, for short dataset partitioned in two files (eg power_consumption ones) it is possible to concatenate them and add artificial faults

%% crop and reshape flowStream1 in a matrix having changeParams.N columns
flowStream1 = flowStream1(1 :  changeParams.N * floor(length(flowStream1) / changeParams.N));
flowStream2 = flowStream2(1 :  changeParams.N * floor(length(flowStream2) / changeParams.N));

% prepare DataSet matrices
DataSet1 = NaN(numel(flowStream1) / changeParams.N, changeParams.N);
DataSet2 = NaN(numel(flowStream2) / changeParams.N, changeParams.N);

for ii = 1 : size(DataSet1, 1)
    % initial stationary state of T_star samples
    DataSet1(ii, :) =  flowStream1((ii - 1) * N + 1 : ii * N);
    DataSet2(ii, :) =  flowStream2((ii - 1) * N + 1 : ii * N);
end

% shift DataSet1 and assign to DataSet2
if size(DataSet1, 1) > 1
    switch(lower(changeParams.changeType))
        case {'offset', 'offset+gain', 'offsetgain', 'gain+offset', 'gain', 'stackat', 'sensordegradation'}
            shiftedRows = [1 : size(DataSet1, 1)];
            shiftedRows = circshift(shiftedRows, [0,-1])';
            %             DataSet2(1, :) = DataSet1(2, :);
            %             DataSet2(2, :) = DataSet1(3, :);
            %             DataSet2(3, :) = DataSet1(1, :);
            DataSet2 = DataSet1(shiftedRows, :);
            DataSet2(end, :) = [];
            DataSet1(end, :) = [];
    end
end

% figure(3241), plot(flowStream1(1 : 12000)), hold on,plot([DataSet1(1, :), DataSet2(1, :)], 'r.'), hold off
for ii = 1 : size(DataSet1, 1)
    % initial stationary state of T_star samples
    DataSet(ii, 1 : N) = DataSet1(ii, :);
    
    % synthtetically add N - T_star samples after the change
    switch(lower(changeParams.changeType))
        case 'source' % change the source generating data, though the period remains the same
            DataSet(ii, N + 1 : 2 * N) = DataSet2(ii, :);
        case {'sourceandrescale', 'source+rescale'} % change the source generating data, the period remains the same and rescale the second source to meet the same mean of the first
            DataSet(ii, N + 1 : 2 * N) = DataSet2(ii, :) - mean(DataSet2(ii, :));
            DataSet(ii, N + 1 : 2 * N) = DataSet(ii, N + 1 : 2 * N) / std(DataSet2(ii, :));
            DataSet(ii, N + 1 : 2 * N) = DataSet(ii, N + 1 : 2 * N) * std(DataSet1(ii, :)) + mean(DataSet1(ii, :));
            %             DataSet(ii, N + 1 : 2 * N) = DataSet(ii, N + 1 : 2 * N) + mean(DataSet1(ii, :));
        case 'offset'
            DataSet(ii, N + 1 : 2 * N) = DataSet2(ii, :) + changeParams.offset * mean(DataSet(ii, 1 : N));
        case {'sensordegradation', 'sensor degradation'}
            DataSet(ii, N + 1 : 2 * N) = DataSet2(ii, :) + changeParams.sensorDegradationStd * mean(DataSet(ii, 1 : N)) * randn(size(DataSet2(ii, :)));
        case 'gain'
            DataSet(ii, N + 1 : 2 * N) = DataSet2(ii, :) * changeParams.gain;
        case {'offset+gain', 'offsetgain', 'gain+offset'}
            DataSet(ii, N + 1 : 2 * N) = DataSet2(ii, :) * changeParams.offset + changeParams.offset  * mean(DataSet(ii, 1 : N));
        case 'stackat'
            DataSet(ii, N + 1 : 2 * N) = DataSet2(ii, 1);
        otherwise
            error([changeParams.changeType, ' not recognized']);
    end
end

% add small noise perturbation
DataSet = DataSet + changeParams.sigma_noise * mean(DataSet(:)) * randn(size(DataSet));

%% remove last rows of the Dataset (eventually)
rowsToRemove = any(isnan(DataSet), 2);
DataSet(rowsToRemove, :) = [];

%% show DataSet (for debug purposes)
if do_show
    figNr = figure(),
    
    for ii = 1 : size(DataSet, 1)
        plot(DataSet(ii, :), 'r.')
        axis tight
        a = get(gca, 'Ylim');
        hold on
        plot([N, N], a, 'm-');
        hold off
        disp('hit a key')
        pause();
    end
end


% for ii = 1 : size(DataSet1, 1)
%     % initial stationary state of T_star samples
%     DataSet(ii, 1 : T_star) = DataSet1(ii, 1 : T_star);
%
%     % synthtetically add N - T_star samples after the change
%     switch(lower(changeParams.changeType))
%         case 'source' % change the source generating data, though the period remains the same
%             DataSet(ii, T_star + 1 : N) = DataSet2(ii, T_star + 1 : N) ;
%         case {'sourceandrescale', 'source+rescale'} % change the source generating data, the period remains the same and rescale the second source to meet the same mean of the first
%             DataSet(ii, T_star + 1 : N) = flowStream2((ii - 1) * N + T_star + 1 : ii * N) - mean(flowStream2((ii - 1) * N + T_star + 1 : ii * N));
%             DataSet(ii, T_star + 1 : N) = DataSet(ii, T_star + 1 : N) / std(flowStream2((ii - 1) * N + T_star + 1 : ii * N));
%             DataSet(ii, T_star + 1 : N) = DataSet(ii, T_star + 1 : N) * std(DataSet(ii, 1 : T_star)) + mean(DataSet(ii, 1 : T_star));
%         case 'offset'
%             DataSet(ii, T_star + 1 : N) = flowStream1((ii - 1) * N + T_star + 1 : ii * N) + changeParams.offset;
%         case 'gain'
%             DataSet(ii, T_star + 1 : N) = flowStream1((ii - 1) * N + T_star + 1 : ii * N)  * changeParams.offset;
%         case {'offset+gain', 'offsetgain', 'gain+offset'}
%             DataSet(ii, T_star + 1 : N) = flowStream1((ii - 1) * N + T_star + 1 : ii * N) * changeParams.offset + changeParams.offset;
%         case 'stackat'
%             DataSet(ii, T_star + 1 : N) = flowStream1((ii - 1) * N + T_star + 1) ;
%     end
% end