%  This is a demo file illustrating the usage of the change-detection test (CDT)
%  exploiting self-similarity as presented  in [1]
%
%
% Giacomo Boracchi
% Politecnico di Milano
% July 2014
% giacomo.boracchi@polimi.it
%
% References
%
% [1] Giacomo Boracchi and Manuel Roveri,
% "Exploiting Self-Similarity for Change Detection"
% in Proceedings of IJCNN 2014, International Joint Conference on Neural Networks
% Beijing, China 5 - 11 July 2014. [Preprint available online at http://home.deib.polimi.it/boracchi/publications.html
%
% [2]  Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "A just-in-time adaptive classification system based on the intersection of confidence intervals rule,"
% Neural Networks, Elsevier vol. 24 (2011), pp. 791-800 doi: 10.1016/j.neunet.2011.05.012


close all
clear
clc


% add the ICI-based CDT Library, which is in a separate package and it's required
% SET the proper path
addpath(genpath('..\2012_Hierarchical_ICI_based_CDT\HierarchicalICI-basedCDT\')); 

addpath('Data');
addpath('functions');

% check if the ICI-based CDT has been properly loaded
strTemp =  which('define_ICI_test_parameters');

if isempty(strTemp)
    error('ICI-based CDT package not found: download from http://home.deib.polimi.it/boracchi/Projects/ and set the proper folder at the line 29');
end


%% Specify the paramters of the BCN WDN database
nPeriodsTS = 14; % number of periods in the training set
period = 144; % number of samples in each period

selfSimilarityParams = define_Self_Similarity_parameters(period, nPeriodsTS);

% this is the change-time instant in each sequence, i.e. T*
changeParams.N = selfSimilarityParams.period * 41 % in [1] Dataset were prepared using 41 periods in each concept

% percentage about the mean of the signal in stationary condition
changeParams.offset = 0.5; % to be used only when changeParams.changeType == 'offset';
changeParams.gain= 1;
changeParams.changeType = 'offset'; % 'offset', 'sensordegradation','gain', s'offsetgain', 'stackat'
changeParams.sigma_noise = 0.0; % AWGN std is mean of the signal in stationary condition times changeParams.sigma_noise
changeParams.sensorDegradationStd = 0.5; % to be used only when changeParams.changeType == 'sensordegradation';
changeParams.changeString = createChangeParamsString(changeParams)


%% CDT parameters
cdtTsLength = 400; % training set for the ICI-based CDT, expressed in terms of change indexes
cdtParams = define_ICI_test_parameters(cdtTsLength);
cdtParams.do_validate_changes = 0; % no hierarchal CDT to be used
cdtParams.do_stop_first_detection = 1;% no need to reconfigure the CDT after the first detection [not implemented]
cdtParams.Gamma = 2.5;

if cdtParams.do_validate_changes == 0
    cdtParams.MinimumTS_Size = 0;
end

% procedural parameters
do_show = 1;

%% create dataset  (10 sequences of flow measurements modified according to changeParams)
[DataSet] = loadDataSet(changeParams);

for ii = 1 : size(DataSet, 1)
    z = DataSet(ii, :);
    
    % runs the cdt on self similarity
    results = f_ICI_CDT_self_similarity(z, selfSimilarityParams, cdtParams);
   
    % remove the samples used for builing the Patch training set P
    z(1 : selfSimilarityParams.signalTS_len) = [];
    
    T_star = changeParams.N - selfSimilarityParams.signalTS_len;
    % tsEnd is a vector containing the end of each training set. 
    % tsEnd(1) is actually the last sample of the initial training set provided for configuring the CDT and has to be
    % ignored. tsEnd(2) is the first detection of the CDT. When numel(results.tsEnd) == 1 no change was detected
    if numel(results.tsEnd) > 1
        T_hat(ii) = results.tsEnd(2) - selfSimilarityParams.signalTS_len;
    else
        T_hat(ii) = -1;
    end
    
    disp(['Detections of the ICI-based CDT at the first layer: ', num2str(T_hat(ii)) , ' true change at T* = ', num2str(T_star)])
    if cdtParams.do_validate_changes
        disp(['   (these detections have been validated by the second layer)']);
        disp(['The detections of the hierarchical CDT are: ', num2str(results.tsEnd(2 : end) - selfSimilarityParams.signalTS_len), ' (possibly taking into account delay for validation purposes)']);
        disp(['Refined estimate of the change-time instant ', num2str(results.tsInit(2 : end) - selfSimilarityParams.signalTS_len)]);
    end

    %% display results
    if do_show
	
	% display Detection results
	LN_WDT = 2;
	FNT_SZ1 = 22;
	FNT_SZ2 = 18;
        
        xRange = max(T_hat(ii), T_star) + 200;  % used to prepare paper images
        
        if isnan(xRange)
            xRange =[numel(z)];
        elseif xRange < cdtTsLength + 200
            xRange = T_star + 2000;
        end
        
        if ~exist('figNrImg', 'var')
            figNrImg= figure();
        else
            figure(figNrImg);
        end
        
        subplot(2, 1, 1),
        plot(z), title(['Observations, change at ', num2str(T_star), ' sequence ', num2str(ii) ], 'FontSize', FNT_SZ1);
        axis tight
        a = get(gca);
        hold on
        range = a.YLim;
        LL(1) = plot([T_star, T_star], range, 'm-' , 'LineWidth' , LN_WDT);
        LL(2)  = plot([T_hat(ii), T_hat(ii)], range, 'r-' , 'LineWidth' , LN_WDT);
        LL(3) = plot([1 : cdtTsLength], z(1 : cdtTsLength), 'g-', 'LineWidth', LN_WDT);
        hold off
        set(gca, 'FontSize', FNT_SZ1);
        legend(LL, 'True leak time', 'Detection ', 'CDT training set')
        xlim([0, xRange]);
        
        subplot(2, 1, 2), plot(results.centersDistance,'k.'),
        title(['distance between patch centers T^ = ', num2str(T_hat(ii))], 'FontSize', FNT_SZ1);
        axis tight
        a = get(gca);
        hold on
        range = a.YLim;
        LL(1)= plot([T_star, T_star], range, 'm-' , 'LineWidth' , LN_WDT);
        LL(2)  = plot([T_hat(ii), T_hat(ii)], range, 'r-' , 'LineWidth' , LN_WDT);
        LL(3) = plot([1 : cdtTsLength], results.centersDistance(1 : cdtTsLength), 'g-', 'LineWidth', LN_WDT);
        hold off
        set(gca, 'FontSize', FNT_SZ1);
        xlim([0, xRange]);
        

        
        %% compute and plot the distributions before and after T^*
        
         if ~exist('figNrHist', 'var')
            figNrHist= figure();
        else
            figure(figNrHist);
         end
        
        featuresBefore = results.centersDistance(1 : T_star);
        featuresAfter =  results.centersDistance(T_star + 1 : end);
        [ksDistance, figNrCenters] = plotHistComparison(featuresBefore, featuresAfter, 'centers Distance ', figNrHist);
        
        
        %% autocorrelation plot
        if ~exist('figXcorr', 'var')
            figXcorr= figure();
        else
            figure(figXcorr);
        end
        
        [r, lags]=xcorr(featuresBefore, 'coeff');
        plot(lags(lags >= 0), r(lags >= 0), 'r.');
        grid on
        xlim([0, 2 * selfSimilarityParams.period + 10])
        ylim([-1,1])
        title('autocorr features')
  
    end
    
end

%%
DD = mean(T_hat(T_hat >= T_star)) - T_star
FPR = nnz(T_hat < T_star) / size(DataSet, 1)