function [centersDistance, patchDistance, closestPoint] = computePatchDissimilarity(z, tsMatrix, tsCenters, selfSimilarityParams, do_debug)
%
% function [centersDistance, patchDistance, closestPoint] = computePatchDissimilarity(z, tsMatrix, tsCenters, selfSimilarityParams, do_debug)
%
% compute the change indicators are described in [1]
% 
% input
% z                                   data sequence to be analyzed
% tsMatrix                        matrix representing the training set of Patches, (P in[1]). Given by buildPatchReferenceMatrix
% tsCenters                      array representing the location of the centers for the patches in P. Given by buildPatchReferenceMatrix
% selfSimilarityParams   parameters for comuting the change indicators. Given by define_Self_Similarity_parameters
%
% output
% centersDistance   change indicators x in [1]
% patchDistance      distance between the current patch and the most similar patch in the training set of patches 
%                               (a different change indicator, not to be used for the ICI-based CDT using SelfSimilarity)
% closestPoint          location of the most similar patch
%
% References
%
% [1] Giacomo Boracchi and Manuel Roveri,
% "Exploiting Self-Similarity for Change Detection"
% in Proceedings of IJCNN 2014, International Joint Conference on Neural Networks
% Beijing, China 5 - 11 July 2014. [Preprint available online at http://home.deib.polimi.it/boracchi/publications.html
%
%
%
% Giacomo Boracchi
% Politecnico di Milano
% January 2014
% giacomo.boracchi@polimi.it
%
% REVISION HISTORY
% July  2014     Engineering
%
%

do_windowing = selfSimilarityParams.do_windowing;
do_fully_non_local = selfSimilarityParams.do_fully_non_local;
SIGMA_WINDOWING = selfSimilarityParams.sigmaWindowing;
HPS = selfSimilarityParams.HPS;
signalTS_len = selfSimilarityParams.signalTS_len;
do_normalize_patches = selfSimilarityParams.do_normalize_patches;
period = selfSimilarityParams.period; % to be used only when do_fully_nonlocal == 0
patchSize = selfSimilarityParams.patchSize;

N = numel(z);
tsMatrixSize = size(tsMatrix, 1);
% require the signal period: reference for SR
tsCentersMod = mod(tsCenters, selfSimilarityParams.period); % patch centers modulo the period

%% compute nonlocal dissimilarity features
patchDistance = NaN(1, N - HPS);
centersDistance  = NaN(1, N - HPS);
closestPoint  = NaN(1, N - HPS);

if do_windowing
    % build a 2D Gaussian kernel
    windowKernel = fspecial('Gaussian', patchSize, SIGMA_WINDOWING);
    % take the line in the middle
    windowKernel = windowKernel(HPS + 1, :);
    % perform normalization such that 1 is assigned to the central pixel
    windowKernel = windowKernel ./ max(windowKernel(:));
    W = windowKernel(ones(1, tsMatrixSize), :);
else
    W = ones(tsMatrixSize, patchSize);
end

% compute change-indicator statistics (i.e., features)
for ii = signalTS_len + 1 : N - HPS
    
    % current patch center is ii
    currentPatch = z(ii - HPS : ii + HPS);
    
    if do_normalize_patches
        % crop a patch and normalize it
        currentPatch = currentPatch - mean(currentPatch);
    end
    
    % replicate patch to have a matrix
    currentMatrix = currentPatch(ones(1, tsMatrixSize), :);
    
    % compute ell2 norm of the difference
    distances = sqrt(sum((W .* (currentMatrix - tsMatrix)).^2, 2));
    
    % determine the subset of time instants in the TS where to locate the patch center
    if do_fully_non_local
        msk = ones(size(distances));
        distanceIndexes = find(msk);
    else
       % compute the indexes of the serach region corresponding to ii in the training set
       SNIndexes = determineSearchNeighIndexes(ii, selfSimilarityParams);
       % consider the search indexes in the training set only
       SN_good_indexes = intersect(tsCenters, SNIndexes); 
       
       % refer this indexes to the distance vector which is shifted by HPS
       distanceIndexes = SN_good_indexes - tsCenters(1) + 1; 
       
%        cen =  cen + [0 : selfSimilarityParams.period : selfSimilarityParams.signalTS_len - selfSimilarityParams.period];
%        
%        msk = zeros(1, selfSimilarityParams.signalTS_len);
%        msk(cen)  = 1;
%        msk = convn(msk', searchNeighFilter, 'same');
%        mskIndexes = find(msk);
%     distanceIndexes = intersect(mskIndexes, tsCenters) - HPS;
       
%         msk = zeros(size(distances));
%         % distance masking to remove pixels out of the temporal range
%         % determine the current time instant
%         tsReferenceStartAt = mod(ii, period); % it does not work HPS should be added here!
%         msk(tsCentersMod == tsReferenceStartAt) = 1;
%         msk = (convn(msk, searchNeighFilter, 'same'));
%         distanceIndexes = find(msk > 0) ;
    end
    
    
    % dissimilarity measure the minimum of patch distance (windoed ell2 norm of patches)
    [patchDistance(ii), closestPoint_temp] = min(distances(distanceIndexes));
    % map the closest point on the mask and then on the center of the patches in the ts
    closestPoint(ii) = tsCenters(distanceIndexes(closestPoint_temp));
    
    % compute distance between centers of closest patches
    centersDistance(ii) = z(ii) - z(closestPoint(ii));
    
    if do_debug  && mod(ii, 500) == 0 % &&  ii >changeParams.N - 100
        MRK_SZ = 10;
        LN_WDT = 3;
        figure(1231), subplot(2,1,1)
        plot(z(1 : ii + HPS))
        hold on
        plot([1 : signalTS_len], z([1 : signalTS_len]), 'g-', 'LineWidth', LN_WDT);
        plot([ii - HPS : ii + HPS], z([ii - HPS : ii + HPS]), 'k-', 'LineWidth', LN_WDT);
        plot(ii, z(ii), 'rx', 'LineWidth', LN_WDT, 'MarkerSize', MRK_SZ);
        plot(SN_good_indexes, z(SN_good_indexes), 'ro', 'LineWidth', 1);
        a = get(gca, 'xlim')
        
       
        plot([closestPoint(ii) - HPS : closestPoint(ii) + HPS], z([closestPoint(ii) - HPS : closestPoint(ii) + HPS]), 'm-', 'LineWidth', LN_WDT);
        grid on
        hold off
        
        l = legend('data', 'trainig set', 'current patch',  'current point', 'centers in Search Region', 'closest Patch');
        set(l, 'Location', 'NorthWest');
        subplot(2,1,2), plot(centersDistance(1 : ii), 'k.'), grid on, title('pixel-wise distance');
        set(gca, 'xlim', a);
        hold on

        hold off
        set(1231, 'Position', [126 1 1315 824])
        saveas(1231, ['temp', num2str(ii),'.png']);

    end
end
