function detections = f_ICI_CDT_self_similarity(z, selfSimilarityParams, cdtParams)
%
%
% function detections = f_ICI_CDT_self_similarity(z, selfSimilarityParams, cdtParams)
%
% Runs the ICI-based CDT exploiting Self similarity as described in [1]
%
% output structure N.B: when the ICI-based CDT does not perform validation and reconfiguration
% the change is detected at detections.tsEnd(2) (the same as detections.confirmedDetections)
%
% detections.tsEnd:             vector indicating the last samples of the training set. 
%                                          It contains the initial training set as well as those estimated during 
%                                          the operational life by the hierarchical ICI-based CDT for both validation 
%                                          and reconfiguration (these are detections of the cdt that have been 
%                                          possibly delayed for gathering enough samples for validation)
% detections.tsInit:              as detections.tsEnd, this is the vector of the first sample of the training sets
% detections.confirmedDetections:     vector containing the detections that have been confirmed by the CDT
% detections.tsLengths:                        vector indicating the lentgth of the training sets, both the 
%                                                           first and those estimated during the operational life
% detections.originalDetections          vector indicating the detections before validating the change (these are not delayed in case validation is required)
% detections.centersDistance              vector of change indicators used detecting the change
%
%
% References
%
% [1] Giacomo Boracchi and Manuel Roveri,
% "Exploiting Self-Similarity for Change Detection"
% in Proceedings of IJCNN 2014, International Joint Conference on Neural Networks
% Beijing, China 5 - 11 July 2014. [Preprint available online at http://home.deib.polimi.it/boracchi/publications.html
%
%
% [2]  Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "A just-in-time adaptive classification system based on the intersection of confidence intervals rule,"
% Neural Networks, Elsevier vol. 24 (2011), pp. 791-800 doi: 10.1016/j.neunet.2011.05.012
%
%
% [3] Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "A Hierarchical, Nonparametric Sequential Change-Detection Test"
% in Proceedings of IJCNN 2011, the International Joint Conference on Neural Networks,
% San Jose, California July 31 - August 5, 2011. pp 2889 - 2896, DOI: 10.1109/IJCNN.2011.6033600
%
%
% preprints available at http://home.deib.polimi.it/boracchi/publications.html
%
%
% Giacomo Boracchi
% Politecnico di Milano
% January 2014
% giacomo.boracchi@polimi.it
%
% REVISION HISTORY
% July  2014     Engineering
%
%


do_debug = 0;

% this does not require an estimate of the signal period
[tsMatrix, tsCenters] = buildPatchReferenceMatrix(z, selfSimilarityParams);

%% compute the distances centersDistance length is z - half patch size
[centersDistance, patchDistance, closestPoint] = computePatchDissimilarity(z, tsMatrix, tsCenters, selfSimilarityParams, do_debug);

%% remove the patch training set 
z(1 : selfSimilarityParams.signalTS_len) = [];

patchDistance(1 : selfSimilarityParams.signalTS_len) = [];
centersDistance(1 : selfSimilarityParams.signalTS_len) = [];
closestPoint(1 : selfSimilarityParams.signalTS_len) = [];

if any(isnan([centersDistance, patchDistance, closestPoint]))
    error('NaN found in indicator metrices')
end

% disp(['change artificially induced at ', num2str(T_star_updated), ' changeType ', num2str(changeParams.changeType)])

%%  run ICI-based CDT

% run ICI-based CDT on the centers distance
[tsEndCenters, tsInitCenters, ~, tsLengths, testDetectingChange, detectionsCenters] = f_ICI_test_MultiChange_MultiStage(centersDistance, cdtParams.tsLength, cdtParams);

% add half a patch and the portion of TS used for computing the self similarity that has been removed 
originalDetections = detectionsCenters + selfSimilarityParams.HPS + selfSimilarityParams.signalTS_len;
% confirmedDetections = confirmedDetections + selfSimilarityParams.HPS + selfSimilarityParams.signalTS_len;
tsInit = tsInitCenters + selfSimilarityParams.HPS + selfSimilarityParams.signalTS_len;
tsEnd = tsEndCenters + selfSimilarityParams.HPS + selfSimilarityParams.signalTS_len;

%% build the output structure
detections.tsEnd = tsEnd; % vector indicating the last samples of the training set. It contains the initial training set as well as those estimated during the operational life by the hierarchical ICI-based CDT for both validation and reconfiguration (these are detections of the cdt that have been possibly delayed for gathering enough samples for validation)
detections.tsInit = tsInit; %, as detections.tsEnd, this is the vector of the first sample of the training sets
detections.tsLengths = tsLengths; % vector indicating the lentgth of the training sets, both the first and those estimated during the operational life
detections.testDetectingChange = testDetectingChange;
detections.originalDetections = originalDetections; % vector indicating the detections before validating the change (these are not delayed in case validation is required)
detections.centersDistance = centersDistance; % vector of change indicators used detecting the change

% code to be used for performing the template-based approach
% % require the signal period: template-based approach
% %[signalTemplate, zForTemplate] = buildAverageTemplate(z, selfSimilarityParams.period, nPeriodsTS, 0);
% 
% % distanceFromTemplate = computeDistanceFromTemplate(z, signalTemplate, selfSimilarityParams);
% 
% % distanceFromTemplate(1 : selfSimilarityParams.signalTS_len) = [];