function [DataSet, figNr] = insertChange(DataSet1, changeParams, do_show)
%
% function [DataSet, figNr] = creaDataSetSignal(fileName1, changeParams, fileName2, do_show)
%
% Generates Dataset for assessing performance of Self-Similarity based CDT. It modifies flow measurements form BCN WDN
%
% input description
%
%
% changeParams structure with fields
% changeParams.N  % length of each sequence of the dataset
% changeParams.T_star  % changePoint of each sequence of the dataset
% changeParams.period  % period expressed as a number of samples
% changeParams.changeType:
%    'source'  % change the source generating data, though the period remains the same (fileName2 required)
%    'sourceAndRescale' % change the source generating data, the period remains the same and rescale the second source to meet the same mean of the first
%    'offset'  % add an offset to the measurements (fileName2 NOT required)
%    'stackat' % stackat at T_star measurements (fileName2 NOT required)
%	 'gain'    % gain change at T_star
%    'offset+gain'
%    'sensor degradation' Add AWGN having std changeParams.sensorDegradationStd * mean flow
%
%
% do_show     % display Dataset (for debug purposes)
%
%
% output description
%
% DataSet   % matrix containing each sequence of changeParams.N data in a row
% figNr     % number of figure used for visualization purposes (only when do_show ~= 1)
%
%
% References
%
% [1] Giacomo Boracchi and Manuel Roveri,
% "Exploiting Self-Similarity for Change Detection"
% in Proceedings of IJCNN 2014, International Joint Conference on Neural Networks
% Beijing, China 5 - 11 July 2014. [Preprint available online at http://home.deib.polimi.it/boracchi/publications.html
%
% [2]  Cesare Alippi, Giacomo Boracchi and Manuel Roveri,
% "A just-in-time adaptive classification system based on the intersection of confidence intervals rule,"
% Neural Networks, Elsevier vol. 24 (2011), pp. 791-800 doi: 10.1016/j.neunet.2011.05.012
%
%
% preprints available at http://home.deib.polimi.it/boracchi/publications.html
%
%
% Giacomo Boracchi
% Politecnico di Milano
% giacomo.boracchi@polimi.it
% 
% Revision History
% January 2014   First Release
% July 2014 Engineering

figNr = [];

if ~exist('do_show', 'var') || isempty(do_show)
    do_show = 0;
end

N = changeParams.N;
DataSet = NaN(size(DataSet1));

%% introduce the change synthetically
for ii = 1 : size(DataSet1, 1)
    % initial stationary state of T_star samples
    DataSet(ii, 1 : N) = DataSet1(ii, 1 : N);
    
    % synthtetically add N - T_star samples after the change
    switch(lower(changeParams.changeType))
        case 'offset'
            DataSet(ii, N + 1 : 2 * N) = DataSet1(ii, N + 1 : 2 * N) + changeParams.offset * mean(DataSet(ii, 1 : N));
        case {'sensordegradation', 'sensor degradation'}
            DataSet(ii, N + 1 : 2 * N) = DataSet1(ii, N + 1 : 2 * N) + changeParams.sensorDegradationStd * mean(DataSet(ii, 1 : N)) * randn(1, N);
        case 'gain'
            DataSet(ii, N + 1 : 2 * N) = DataSet1(ii, N + 1 : 2 * N) * changeParams.gain;
        case {'offset+gain', 'offsetgain', 'gain+offset'}
            DataSet(ii, N + 1 : 2 * N) = DataSet1(ii, N + 1 : 2 * N) * changeParams.offset + changeParams.offset  * mean(DataSet(ii, 1 : N));
        case 'stackat'
            DataSet(ii, N + 1 : 2 * N) = DataSet1(ii, N + 1);
        otherwise
            error([changeParams.changeType, ' not recognized']);
    end
end

% add small noise perturbation
DataSet = DataSet + changeParams.sigma_noise * mean(DataSet(:)) * randn(size(DataSet));

%% remove last rows of the Dataset (eventually)
rowsToRemove = any(isnan(DataSet), 2);
DataSet(rowsToRemove, :) = [];

%% show DataSet (for debug purposes)
if do_show
    figNr = figure(),
    
    for ii = 1 : size(DataSet, 1)
        plot(DataSet(ii, :), 'r.')
        axis tight
        a = get(gca, 'Ylim');
        hold on
        plot([N, N], a, 'm-');
        hold off
        disp('hit a key')
        pause();
    end
end


% for ii = 1 : size(DataSet1, 1)
%     % initial stationary state of T_star samples
%     DataSet(ii, 1 : T_star) = DataSet1(ii, 1 : T_star);
%
%     % synthtetically add N - T_star samples after the change
%     switch(lower(changeParams.changeType))
%         case 'source' % change the source generating data, though the period remains the same
%             DataSet(ii, T_star + 1 : N) = DataSet2(ii, T_star + 1 : N) ;
%         case {'sourceandrescale', 'source+rescale'} % change the source generating data, the period remains the same and rescale the second source to meet the same mean of the first
%             DataSet(ii, T_star + 1 : N) = flowStream2((ii - 1) * N + T_star + 1 : ii * N) - mean(flowStream2((ii - 1) * N + T_star + 1 : ii * N));
%             DataSet(ii, T_star + 1 : N) = DataSet(ii, T_star + 1 : N) / std(flowStream2((ii - 1) * N + T_star + 1 : ii * N));
%             DataSet(ii, T_star + 1 : N) = DataSet(ii, T_star + 1 : N) * std(DataSet(ii, 1 : T_star)) + mean(DataSet(ii, 1 : T_star));
%         case 'offset'
%             DataSet(ii, T_star + 1 : N) = flowStream1((ii - 1) * N + T_star + 1 : ii * N) + changeParams.offset;
%         case 'gain'
%             DataSet(ii, T_star + 1 : N) = flowStream1((ii - 1) * N + T_star + 1 : ii * N)  * changeParams.offset;
%         case {'offset+gain', 'offsetgain', 'gain+offset'}
%             DataSet(ii, T_star + 1 : N) = flowStream1((ii - 1) * N + T_star + 1 : ii * N) * changeParams.offset + changeParams.offset;
%         case 'stackat'
%             DataSet(ii, T_star + 1 : N) = flowStream1((ii - 1) * N + T_star + 1) ;
%     end
% end